' Program: SkyRC.bas
' Version: 3.0
' Date:    4 November 2003
'
' PROGRAM COMPILED OK. 386 Words used
' 25 Variables used in the 16F877 from a possible 368
'
' This program performs a decode of the BSkyB 'Open' keyboard and BSkyB Remote Control Handsets
' I use a PIC 16F877 chip running at 4MHz in the Proton Mk2 development board.
' I worked out the delay times using a 20 MHz Oscilloscope and also by using Timer1 to measure the pulse widths.
' The delay times for a Sky Remote Control Handset appear to be a bit longer.
' I measured the average 1/2 bit times as:
'
' 	  			  	  	 	  	   410 microseconds for the Keyboard
'								   420 microseconds for the Remote Control Handset
'
' The pulses do not appear to have a 50:50 duty cycle and I have found that the initial delay I used could vary from
' 1880 microseconds to as much as 2100 microseconds with reliable results. However, the half bit delay time only worked
' when set to 410 microseconds for the keyboard and 420 microseconds for the remote control handset. 
' NB: When using the keyboard or remote - beware of pressing the "TV/Sat" button as this changes the keyboard protocol
'     from sky digibox to tv and I have not programmed this yet so I ignore the TV codes.
' The program outputs to the Serial Terminal at 9600 baud. It gives the source (K=Keyboard or R=Remote) and decimal value 
' of the command received. e.g. <K 101> means Keyboard Command 101 and <R 0> means Remote Control Handset Command 0
' It also flashes the LEDs with the Command Value.
'
'-----------------------------------------------------------------------------------------------------------------------------
'
' The structure of the command codes is as follows:
'
' Quiescent   	         ..1111110  		 		   	 Signal High - Wait until pin goes low
' Preamble		         00000					  	     Wait 5 x 410us = 2050us, effectively skipping over the preamble
' Header or Device ID    0011 0101 0110 1100		  	 $35 6C
' Unused by BSkyB 	 	 1010 1010 1010 1010 1010 1010   $AA AA AA  Manchester Encoding, %10 = 0 and %01 = 1, i.e. all Zeroes
' Command Value 	 	 1010 1010 1010 1010 	  		 $AA AA Manchester Encoding $AAAA = %00000000 e.g. Key 0 on Remote			
' Quiescent			 	 11111111..........				 Signal High - approximately 120 milliseconds between each command
'
' So the Coded Command consumes 7 bytes (56 bits following preamble) which we will read and store in the 7 byte 
' buffer area before Parsing. 
'-----------------------------------------------------------------------------------------------------------------------------
		Include "PROTON_4.INC"

		Dim Buffer1 as Byte SYSTEM	   	   ' Input buffer for data	(SYSTEM keeps it in BANK 0)
		Dim Buffer2 as Byte SYSTEM		   ' Consists of Buffer1 to Buffer7 :  7 bytes = 56 bits
		Dim Buffer3 as Byte SYSTEM		   ' 		  2 bytes of Header or Device ID
		Dim Buffer4 as Byte SYSTEM		   '		  3 bytes of Unused space - all set to $AA (== Zeroes)
		Dim Buffer5 as Byte SYSTEM		   '		  2 bytes of Command Value - 16 bits of Manchester Encoded data
		Dim Buffer6 as Byte SYSTEM		   '		  		  	 where %10 == 0 and %01 == 1
		Dim Buffer7 as Byte SYSTEM		   ' So Command Value is 1 byte containing values from 0..255 inclusive
		
		Dim DelayTime as Word			   ' Half bit delay time
		Dim BitCount as Byte	   		   ' Counter
		Dim BitPosn	as Byte				   ' Counter
		Dim Command as Byte				   ' Used to capture and decode the Command Value byte
		Dim Keyboard as Byte			   ' R when Remote Handset is used, K when Open Keyboard is used
        Symbol Signal = PORTC.0     	   ' Input data pin from Infrared Receiver
        Symbol CARRY = STATUS.0			   ' Carry flag
		Symbol LED = PortD
				
		Symbol KbdDelay = 410			   ' 410 microseconds per bit (half bit in Manchester Coded pair) 
		Symbol RemDelay = 420              ' 420 microseconds per bit (half bit in Manchester Coded pair)
		                       
        Delayms 500						   ' Wait for PICmicro to settle down
		TRISD = %00000000
        INTCON = 0						   ' Clear int-flags, Disable interrupts


'		DelayTime = KbdDelay			   ' Start with Open Keyboard timings
		DelayTime = RemDelay			   ' Start with Remote Control Handset timings

		if DelayTime = KbdDelay then
		   Keyboard = "K"
		else
		   Keyboard = "R"
		endif

Main:
	    Delayms 200	  					   ' Slows down the repeat rate and allows LEDS to be seen to flash
		LED = 0							   ' switch LEDs off again

Main1:
        Clear BitCount
        Clear BitPosn	
        FSR = VARPTR(Buffer1)			   ' Set the indirect pointer to the start of the buffer space
		While Signal = 1: Wend			   ' Wait until first IR signal (goes low) is received
		Delayus 2050 	  				   ' Debounce - ignoring 5 bits of preamble which are all low
										   ' Delay = 5 * 410 = 2050
										   ' Experiment shows anything from 1880 to 2100 gives reliable results
										   
		If Signal = 1 Then Main1		   ' Check for signal, it should still be be low
		   			   	   				   ' If it's high, try again
										    
		While 1 = 1						   ' Create a loop for 56 bits
			Set CARRY					   ' Preset the carry bit
        	If Signal = 0 Then Clrc 	   ' Test Signal
										   ' Input bit is low so change carry
										   ' Now the input bit is in the carry flag
			Rol	INDF					   ' Rotate carry into the buffer
			Inc BitCount 				   ' Increment the bit counter
        	If BitCount > 55 Then Decode   ' See if we've done all 56 bits
			INC	BitPosn	 
			If BitPosn.3 = 1 Then		   ' See if all 8 bits are filled 
        		Inc	FSR					   ' If Yes, use next buffer byte
				Clear BitPosn 			   ' and reset bit position counter
        	Endif
			Delayus DelayTime			   ' Wait half bit before testing signal again
        Wend 							   ' Loop back & do the rest of the bits

Decode:
        FSR = VARPTR(Buffer1)			   ' Reset the indirect pointer to the start of the buffer space

Decode1:	  							   ' Check 1st Byte
		if indf = $35 then Decode2   	   ' Looking for &35 in byte 1 i.e. bit pattern %00110101
		goto Main1 				 		   ' It isn't so try again

Decode2:   		 	  	  		   		   ' Check 2nd Byte
		inc FSR	
		if indf = $6C then Decode3   	   ' Looking for $6C in byte 2 i.e. bit pattern %01101100
		gosub KeyboardCheck		 		   ' If it isn't, switch between keyboard timing and handset timing
		goto Main1	   					   ' Then try again

Decode3:   			 	  		   		   ' Check 3rd Byte
		inc FSR	
		if indf = $AA then Decode4   	   ' Looking for &AA in Byte 3 i.e 4 lots of %10 = 4 Zeroes
		gosub KeyboardCheck		 		   ' If it isn't, switch between keyboard timing and handset timing
		goto Main1

Decode4:   			 	  		   		   ' 
		inc FSR							   ' Check 4th Byte
		if indf <> $AA then Main1  	   	   ' Looking for $AA in Byte 4 i.e 4 lots of %10 = 4 Zeroes

Decode5:	 	 	   	   				   '  
		inc FSR	   			 	  		   ' Check 5th Byte
		if indf <> $AA then Main1	   	   ' Looking for &AA in Byte 5 i.e 4 lots of %10 = 4 Zeroes
		
		inc FSR	   			 	  		   ' Jump over byte 6 because we want to decode byte 7 first
		inc FSR							   ' Now pointing at Byte 7
		
		   		   			 	  		   ' Decode Bytes 7 then 6 which contain the command id
		Clear Command					   ' Clear  Command - this is where the command bits will be assembled
		Clear BitCount					   ' Zeroise the bit counter

CmdLoop:	
		ror indf	  					   ' Data is Manchester encoded with %10 == 0 and %01 == 1
		if CARRY = indf.0 then Main1 	   ' This checks that we don't have an invalid %00 or %11 combination   
		ror Command						   ' Rotate 1st bit from Carry into Command. As we are only interested in the first 
		ror indf					   	   ' bit position, 2nd ror gets rid of the second unwanted bit
		
		inc BitCount					   ' Do this 4 times in Byte 7 then decrement indirect pointer to be
		if BitCount = 4 then dec fsr	   ' pointing at Byte 6.
		
		if Bitcount < 8 then CmdLoop	   ' When we have done this 8 times we have all 8 bits of the command in Command
		
		rsout "<", Keyboard, " ", dec Command, ">" 'Tell the world what we have found
		
		LED = Command
		if LED = 0 then LED = $FF  	  	   ' Flash valid Command on LEDs Command '0' flashes all the LEDs
		    	 	   	   	   	  	  	   ' NB: Command 0 is Remote Key 0 and Command 255 is Keyboard Key 9
		   	  		 	   				   '     So these codes will both flash ALL the LEDs
										    
		Goto Main						   ' Get the next Command
		
KeyboardCheck: 							   ' Subroutine to switch between dealing with the keyboard and a handset
		if Keyboard = "K" then	 		   ' If we are timing the keyboard (KbdDelay for each bit)
		   DelayTime = RemDelay 		   ' then switch to timing for remote control handset
		   Keyboard = "R"				   ' (RemDelay for each bit)
		else		    				   ' and vice versa
		   DelayTime = KbdDelay
		   Keyboard = "K"
		endif
		return
